Package: phylolm
Version: 2.6.5
Date: 2024-09-24
Title: Phylogenetic Linear Regression
Authors@R: c(person("Lam Si Tung", "Ho", role=c("aut", "cre"), email="lamho86@gmail.com"),
            person("Cecile", "Ane", role="aut"),
			person("Robert", "Lachlan", role="ctb"),
			person("Kelsey", "Tarpinian",role="ctb"),
      person("Rachel", "Feldman",role="ctb"),
      person("Qing", "Yu",role="ctb"),
      person("Wouter", "van der Bijl",role="ctb"),
      person("Joan", "Maspons",role="ctb"),
      person("Rutger", "Vos",role="ctb"),
      person("Paul", "Bastide",role="ctb"))
Depends: R (>= 3.0), ape
Imports: future.apply
Description: Provides functions for fitting phylogenetic linear models and phylogenetic generalized linear models. The computation uses an algorithm that is linear in the number of tips in the tree. The package also provides functions for simulating continuous or binary traits along the tree. Other tools include functions to test the adequacy of a population tree.
License: GPL (>= 2) | file LICENSE
URL: https://github.com/lamho86/phylolm
BugReports: https://github.com/lamho86/phylolm/issues
Encoding: UTF-8
Packaged: 2024-09-30 13:59:25 UTC; lamho
NeedsCompilation: yes
Suggests: testthat, nlme
Author: Lam Si Tung Ho [aut, cre],
  Cecile Ane [aut],
  Robert Lachlan [ctb],
  Kelsey Tarpinian [ctb],
  Rachel Feldman [ctb],
  Qing Yu [ctb],
  Wouter van der Bijl [ctb],
  Joan Maspons [ctb],
  Rutger Vos [ctb],
  Paul Bastide [ctb]
Maintainer: Lam Si Tung Ho <lamho86@gmail.com>
Repository: CRAN
Date/Publication: 2024-09-30 19:00:02 UTC
Built: R 4.4.2; x86_64-apple-darwin19.6.0; 2024-12-07 06:46:38 UTC; unix
